@@ -1,12 +1,13 @@
 # coding: utf-8
 """Honeybee Aperture."""
 from ._basewithshade import _BaseWithShade
+from .typing import clean_string
 from .properties import ApertureProperties
-from .aperturetype import aperture_types
 from .boundarycondition import boundary_conditions, Outdoors, Surface
 from .shade import Shade
-import honeybee.writer as writer
+import honeybee.writer.aperture as writer
 
+from ladybug_geometry.geometry2d.pointvector import Vector2D
 from ladybug_geometry.geometry3d.pointvector import Point3D, Vector3D
 from ladybug_geometry.geometry3d.face import Face3D
 
@@ -16,38 +17,39 @@ import math
 class Aperture(_BaseWithShade):
     """A single planar Aperture in a Face.
 
+    Args:
+        identifier: Text string for a unique Aperture ID. Must be < 100 characters and
+            not contain any spaces or special characters.
+        geometry: A ladybug-geometry Face3D.
+        boundary_condition: Boundary condition object (Outdoors, Surface).
+            Default: Outdoors.
+        is_operable: Boolean to note whether the Aperture can be opened for
+            ventilation. Default: False
+
     Properties:
-        name
-        display_name
-        type
-        boundary_condition
-        indoor_shades
-        outdoor_shades
-        parent
-        has_parent
-        geometry
-        vertices
-        upper_left_vertices
-        triangulated_mesh3d
-        normal
-        center
-        area
-        perimeter
+        * identifier
+        * display_name
+        * boundary_condition
+        * is_operable
+        * indoor_shades
+        * outdoor_shades
+        * parent
+        * has_parent
+        * geometry
+        * vertices
+        * upper_left_vertices
+        * triangulated_mesh3d
+        * normal
+        * center
+        * area
+        * perimeter
+        * user_data
     """
-    TYPES = aperture_types
-    __slots__ = ('_geometry', '_parent', '_type', '_boundary_condition')
+    __slots__ = ('_geometry', '_parent', '_boundary_condition', '_is_operable')
 
-    def __init__(self, name, geometry, type=None, boundary_condition=None):
-        """A single planar aperture in a face.
-
-        Args:
-            name: Aperture name. Must be < 100 characters.
-            geometry: A ladybug-geometry Face3D.
-            type: Aperture type. Default: Window
-            boundary_condition: Boundary condition object (Outdoors, Surface).
-                Default: Outdoors.
-        """
-        _BaseWithShade.__init__(self, name)  # process the name
+    def __init__(self, identifier, geometry, boundary_condition=None, is_operable=False):
+        """A single planar aperture in a face."""
+        _BaseWithShade.__init__(self, identifier)  # process the identifier
 
         # process the geometry
         assert isinstance(geometry, Face3D), \
@@ -55,9 +57,9 @@ class Aperture(_BaseWithShade):
         self._geometry = geometry
         self._parent = None  # _parent will be set when the Aperture is added to a Face
 
-        # process the type and boundary condition
-        self.type = type or aperture_types.window
+        # process the boundary condition and type
         self.boundary_condition = boundary_condition or boundary_conditions.outdoors
+        self.is_operable = is_operable
 
         # initialize properties for extensions
         self._properties = ApertureProperties(self)
@@ -73,7 +75,7 @@ class Aperture(_BaseWithShade):
         assert data['type'] == 'Aperture', 'Expected Aperture dictionary. ' \
             'Got {}.'.format(data['type'])
 
-        ap_type = aperture_types.by_name(data['aperture_type'])
+        is_operable = data['is_operable'] if 'is_operable' in data else False
         if data['boundary_condition']['type'] == 'Outdoors':
             boundary_condition = Outdoors.from_dict(data['boundary_condition'])
         elif data['boundary_condition']['type'] == 'Surface':
@@ -82,40 +84,37 @@ class Aperture(_BaseWithShade):
             raise ValueError(
                 'Boundary condition "{}" is not supported for Apertures.'.format(
                     data['boundary_condition']['type']))
-        aperture = cls(data['name'], Face3D.from_dict(data['geometry']),
-                       ap_type, boundary_condition)
+        aperture = cls(data['identifier'], Face3D.from_dict(data['geometry']),
+                       boundary_condition, is_operable)
         if 'display_name' in data and data['display_name'] is not None:
-            aperture._display_name = data['display_name']
+            aperture.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            aperture.user_data = data['user_data']
         aperture._recover_shades_from_dict(data)
+
+        if data['properties']['type'] == 'ApertureProperties':
+            aperture.properties._load_extension_attr_from_dict(data['properties'])
         return aperture
 
     @classmethod
-    def from_vertices(cls, name, vertices, type=None, boundary_condition=None):
+    def from_vertices(cls, identifier, vertices, boundary_condition=None, is_operable=False):
         """Create an Aperture from vertices with each vertex as an iterable of 3 floats.
 
         Args:
-            name: Aperture name. Must be < 100 characters.
+            identifier: Text string for a unique Aperture ID. Must be < 100 characters and
+                not contain any spaces or special characters.
             vertices: A flattened list of 3 or more vertices as (x, y, z).
-            type: Aperture type. Default: Window
             boundary_condition: Boundary condition object (eg. Outdoors, Surface).
                 Default: Outdoors.
+            is_operable: Boolean to note whether the Aperture can be opened for
+                natural ventilation. Default: False
         """
         geometry = Face3D(tuple(Point3D(*v) for v in vertices))
-        return cls(name, geometry, type, boundary_condition)
-
-    @property
-    def type(self):
-        """Object for Type of Aperture (ie. Window)."""
-        return self._type
-
-    @type.setter
-    def type(self, value):
-        assert value in self.TYPES, '{} is not a valid aperture type.'.format(value)
-        self._type = value
+        return cls(identifier, geometry, boundary_condition, is_operable)
 
     @property
     def boundary_condition(self):
-        """Boundary condition of this aperture."""
+        """Get or set the boundary condition of this aperture."""
         return self._boundary_condition
 
     @boundary_condition.setter
@@ -129,29 +128,43 @@ class Aperture(_BaseWithShade):
                                  'condition. Got {}'.format(type(value)))
         self._boundary_condition = value
 
+    @property
+    def is_operable(self):
+        """Get or set a boolean for whether the Aperture can be opened for ventilation.
+        """
+        return self._is_operable
+
+    @is_operable.setter
+    def is_operable(self, value):
+        try:
+            self._is_operable = bool(value)
+        except TypeError:
+            raise TypeError(
+                'Expected boolean for Aperture.is_operable. Got {}.'.format(value))
+
     @property
     def parent(self):
-        """Parent Face if assigned. None if not assigned."""
+        """Get the parent Face if assigned. None if not assigned."""
         return self._parent
 
     @property
     def has_parent(self):
-        """Boolean noting whether this Aperture has a parent Face."""
+        """Get a boolean noting whether this Aperture has a parent Face."""
         return self._parent is not None
 
     @property
     def geometry(self):
-        """A ladybug_geometry Face3D object representing the aperture."""
+        """Get a ladybug_geometry Face3D object representing the aperture."""
         return self._geometry
 
     @property
     def vertices(self):
-        """List of vertices for the aperture (in counter-clockwise order)."""
+        """Get a list of vertices for the aperture (in counter-clockwise order)."""
         return self._geometry.vertices
 
     @property
     def upper_left_vertices(self):
-        """List of vertices starting from the upper-left corner.
+        """Get a list of vertices starting from the upper-left corner.
 
         This property should be used when exporting to EnergyPlus / OpenStudio.
         """
@@ -159,7 +172,7 @@ class Aperture(_BaseWithShade):
 
     @property
     def triangulated_mesh3d(self):
-        """A ladybug_geometry Mesh3D of the aperture geometry composed of triangles.
+        """Get a ladybug_geometry Mesh3D of the aperture geometry composed of triangles.
 
         In EnergyPlus / OpenStudio workflows, this property is used to subdivide
         the aperture when it has more than 4 vertices. This is necessary since
@@ -169,13 +182,13 @@ class Aperture(_BaseWithShade):
 
     @property
     def normal(self):
-        """A ladybug_geometry Vector3D for the direction the aperture is pointing.
+        """Get a ladybug_geometry Vector3D for the direction the aperture is pointing.
         """
         return self._geometry.normal
 
     @property
     def center(self):
-        """A ladybug_geometry Point3D for the center of the aperture.
+        """Get a ladybug_geometry Point3D for the center of the aperture.
 
         Note that this is the center of the bounding rectangle around this geometry
         and not the area centroid.
@@ -184,14 +197,68 @@ class Aperture(_BaseWithShade):
 
     @property
     def area(self):
-        """The area of the aperture."""
+        """Get the area of the aperture."""
         return self._geometry.area
 
     @property
     def perimeter(self):
-        """The perimeter of the aperture."""
+        """Get the perimeter of the aperture."""
         return self._geometry.perimeter
 
+    def horizontal_orientation(self, north_vector=Vector2D(0, 1)):
+        """Get a number between 0 and 360 for the orientation of the aperture in degrees.
+
+        0 = North, 90 = East, 180 = South, 270 = West
+
+        Args:
+            north_vector: A ladybug_geometry Vector2D for the north direction.
+                Default is the Y-axis (0, 1).
+        """
+        return math.degrees(
+            north_vector.angle_clockwise(Vector2D(self.normal.x, self.normal.y)))
+
+    def cardinal_direction(self, north_vector=Vector2D(0, 1)):
+        """Get text description for the cardinal direction that the aperture is pointing.
+
+        Will be one of the following: ('North', 'NorthEast', 'East', 'SouthEast',
+        'South', 'SouthWest', 'West', 'NorthWest').
+
+        Args:
+            north_vector: A ladybug_geometry Vector2D for the north direction.
+                Default is the Y-axis (0, 1).
+        """
+        orient = self.horizontal_orientation(north_vector)
+        orient_text = ('North', 'NorthEast', 'East', 'SouthEast', 'South',
+                       'SouthWest', 'West', 'NorthWest')
+        angles = (22.5, 67.5, 112.5, 157.5, 202.5, 247.5, 292.5, 337.5)
+        for i, ang in enumerate(angles):
+            if orient < ang:
+                return orient_text[i]
+        return orient_text[0]
+
+    def add_prefix(self, prefix):
+        """Change the identifier of this object and all child objects by inserting a prefix.
+
+        This is particularly useful in workflows where you duplicate and edit
+        a starting object and then want to combine it with the original object
+        into one Model (like making a model of repeated rooms) since all objects
+        within a Model must have unique identifiers.
+
+        Args:
+            prefix: Text that will be inserted at the start of this object's
+                (and child objects') identifier and display_name. It is recommended
+                that this prefix be short to avoid maxing out the 100 allowable
+                characters for honeybee identifiers.
+        """
+        self._identifier = clean_string('{}_{}'.format(prefix, self.identifier))
+        self.display_name = '{}_{}'.format(prefix, self.display_name)
+        self.properties.add_prefix(prefix)
+        self._add_prefix_shades(prefix)
+        if isinstance(self._boundary_condition, Surface):
+            new_bc_objs = (clean_string('{}_{}'.format(prefix, adj_name)) for adj_name
+                           in self._boundary_condition._boundary_condition_objects)
+            self._boundary_condition = Surface(new_bc_objs, True)
+
     def set_adjacency(self, other_aperture):
         """Set this aperture to be adjacent to another.
 
@@ -206,30 +273,36 @@ class Aperture(_BaseWithShade):
         """
         assert isinstance(other_aperture, Aperture), \
             'Expected Aperture. Got {}.'.format(type(other_aperture))
+        assert other_aperture.is_operable is self.is_operable, \
+            'Adjacent apertures must have matching is_operable properties.'
         self._boundary_condition = boundary_conditions.surface(other_aperture, True)
         other_aperture._boundary_condition = boundary_conditions.surface(self, True)
 
-    def overhang(self, depth, angle=0, indoor=False, tolerance=0, base_name=None):
+    def overhang(self, depth, angle=0, indoor=False, tolerance=0.01, base_name=None):
         """Add a single overhang for this Aperture.
 
         Args:
             depth: A number for the overhang depth.
             angle: A number for the for an angle to rotate the overhang in degrees.
-                Default is 0 for no rotation.
+                Positive numbers indicate a downward rotation while negative numbers
+                indicate an upward rotation. Default is 0 for no rotation.
             indoor: Boolean for whether the overhang should be generated facing the
                 opposite direction of the aperture normal (typically meaning
                 indoor geometry). Default: False.
             tolerance: An optional value to return None if the overhang has a length less
-                than the tolerance. Default is 0, which will always yeild an overhang.
+                than the tolerance. Default: 0.01, suitable for objects in meters.
             base_name: Optional base name for the shade objects. If None, the default
                 is InOverhang or OutOverhang depending on whether indoor is True.
+
+        Returns:
+            A list of the new Shade objects that have been generated.
         """
         if base_name is None:
             base_name = 'InOverhang' if indoor else 'OutOverhang'
-        self.louvers_by_number(1, depth, angle=angle, indoor=indoor,
-                               tolerance=tolerance, base_name=base_name)
+        return self.louvers_by_count(1, depth, angle=angle, indoor=indoor,
+                                     tolerance=tolerance, base_name=base_name)
 
-    def right_fin(self, depth, angle=0, indoor=False, tolerance=0, base_name=None):
+    def right_fin(self, depth, angle=0, indoor=False, tolerance=0.01, base_name=None):
         """Add a single vertical fin on the right side of this Aperture.
 
         Args:
@@ -240,16 +313,20 @@ class Aperture(_BaseWithShade):
                 opposite direction of the aperture normal (typically meaning
                 indoor geometry). Default: False.
             tolerance: An optional value to return None if the fin has a length less
-                than the tolerance. Default is 0, which will always yeild a fin.
+                than the tolerance. Default: 0.01, suitable for objects in meters.
             base_name: Optional base name for the shade objects. If None, the default
                 is InRightFin or OutRightFin depending on whether indoor is True.
+
+        Returns:
+            A list of the new Shade objects that have been generated.
         """
         if base_name is None:
             base_name = 'InRightFin' if indoor else 'OutRightFin'
-        self.louvers_by_number(1, depth, angle=angle, contour_vector=Vector3D(1, 0, 0),
-                               indoor=indoor, tolerance=tolerance, base_name=base_name)
+        return self.louvers_by_count(
+            1, depth, angle=angle, contour_vector=Vector2D(1, 0),
+            indoor=indoor, tolerance=tolerance, base_name=base_name)
 
-    def left_fin(self, depth, angle=0, indoor=False, tolerance=0, base_name=None):
+    def left_fin(self, depth, angle=0, indoor=False, tolerance=0.01, base_name=None):
         """Add a single vertical fin on the left side of this Aperture.
 
         Args:
@@ -260,15 +337,19 @@ class Aperture(_BaseWithShade):
                 opposite direction of the aperture normal (typically meaning
                 indoor geometry). Default: False.
             tolerance: An optional value to return None if the fin has a length less
-                than the tolerance. Default is 0, which will always yeild a fin.
+                than the tolerance. Default: 0.01, suitable for objects in meters.
             base_name: Optional base name for the shade objects. If None, the default
                 is InLeftFin or OutLeftFin depending on whether indoor is True.
+
+        Returns:
+            A list of the new Shade objects that have been generated.
         """
         if base_name is None:
             base_name = 'InLeftFin' if indoor else 'OutLeftFin'
-        self.louvers_by_number(1, depth, angle=angle, contour_vector=Vector3D(1, 0, 0),
-                               flip_start_side=True, indoor=indoor,
-                               tolerance=tolerance, base_name=base_name)
+        return self.louvers_by_count(
+            1, depth, angle=angle, contour_vector=Vector2D(1, 0),
+            flip_start_side=True, indoor=indoor, tolerance=tolerance,
+            base_name=base_name)
 
     def extruded_border(self, depth, indoor=False, base_name=None):
         """Add a series of Shade objects to this Aperture that form an extruded border.
@@ -276,10 +357,13 @@ class Aperture(_BaseWithShade):
         Args:
             depth: A number for the extrusion depth.
             indoor: Boolean for whether the extrusion should be generated facing the
-                opposite direction of the aperture normal (typically meaning
-                indoor geometry). Default: False.
+                opposite direction of the aperture normal and added to the Aperture's
+                indoor_shades instead of outdoor_shades. Default: False.
             base_name: Optional base name for the shade objects. If None, the default
                 is InBorder or OutBorder depending on whether indoor is True.
+
+        Returns:
+            A list of the new Shade objects that have been generated.
         """
         extru_vec = self.normal if indoor is False else self.normal.reverse()
         extru_vec = extru_vec * depth
@@ -292,24 +376,25 @@ class Aperture(_BaseWithShade):
         for seg in self.geometry.boundary_segments:
             shade_geo = Face3D.from_extrusion(seg, extru_vec)
             extrusion.append(
-                Shade(shd_name_base.format(self.display_name, shd_count), shade_geo))
+                Shade(shd_name_base.format(self.identifier, shd_count), shade_geo))
             shd_count += 1
         if self.geometry.has_holes:
             for hole in self.geometry.hole_segments:
                 for seg in hole:
                     shade_geo = Face3D.from_extrusion(seg, extru_vec)
                     extrusion.append(
-                        Shade(shd_name_base.format(self.display_name, shd_count),
+                        Shade(shd_name_base.format(self.identifier, shd_count),
                               shade_geo))
                     shd_count += 1
         if indoor:
-            self._indoor_shades.extend(extrusion)
+            self.add_indoor_shades(extrusion)
         else:
-            self._outdoor_shades.extend(extrusion)
+            self.add_outdoor_shades(extrusion)
+        return extrusion
 
-    def louvers_by_number(self, louver_count, depth, offset=0, angle=0,
-                          contour_vector=Vector3D(0, 0, 1), flip_start_side=False,
-                          indoor=False, tolerance=0, base_name=None):
+    def louvers_by_count(self, louver_count, depth, offset=0, angle=0,
+                         contour_vector=Vector2D(0, 1), flip_start_side=False,
+                         indoor=False, tolerance=0.01, base_name=None):
         """Add a series of louvered Shade objects covering this Aperture.
 
         Args:
@@ -318,9 +403,13 @@ class Aperture(_BaseWithShade):
             offset: A number for the distance to louvers from this aperture.
                 Default is 0 for no offset.
             angle: A number for the for an angle to rotate the louvers in degrees.
-                Default is 0 for no rotation.
-            contour_vector: A Vector3D for the direction along which contours
-                are generated. Default is Z-Axis, which generates horizontal louvers.
+                Positive numbers indicate a downward rotation while negative numbers
+                indicate an upward rotation. Default is 0 for no rotation.
+            contour_vector: A Vector2D for the direction along which contours
+                are generated. This 2D vector will be interpreted into a 3D vector
+                within the plane of this Aperture. (0, 1) will usually generate
+                horizontal contours in 3D space, (1, 0) will generate vertical
+                contours, and (1, 1) will generate diagonal contours. Default: (0, 1).
             flip_start_side: Boolean to note whether the side the louvers start from
                 should be flipped. Default is False to have louvers on top or right.
                 Setting to True will start contours on the bottom or left.
@@ -328,10 +417,12 @@ class Aperture(_BaseWithShade):
                 opposite direction of the aperture normal (typically meaning
                 indoor geometry). Default: False.
             tolerance: An optional value to remove any louvers with a length less
-                than the tolerance. Default is 0, which will include all louvers
-                no matter how small.
+                than the tolerance. Default: 0.01, suitable for objects in meters.
             base_name: Optional base name for the shade objects. If None, the default
                 is InShd or OutShd depending on whether indoor is True.
+
+        Returns:
+            A list of the new Shade objects that have been generated.
         """
         assert louver_count > 0, 'louver_count must be greater than 0.'
         angle = math.radians(angle)
@@ -345,15 +436,16 @@ class Aperture(_BaseWithShade):
         else:
             shd_name_base = '{}_' + str(base_name) + '{}'
         for i, shade_geo in enumerate(shade_faces):
-            louvers.append(Shade(shd_name_base.format(self.display_name, i), shade_geo))
+            louvers.append(Shade(shd_name_base.format(self.identifier, i), shade_geo))
         if indoor:
-            self._indoor_shades.extend(louvers)
+            self.add_indoor_shades(louvers)
         else:
-            self._outdoor_shades.extend(louvers)
+            self.add_outdoor_shades(louvers)
+        return louvers
 
     def louvers_by_distance_between(
-            self, distance, depth, offset=0, angle=0, contour_vector=Vector3D(0, 0, 1),
-            flip_start_side=False, indoor=False, tolerance=0, base_name=None):
+            self, distance, depth, offset=0, angle=0, contour_vector=Vector2D(0, 1),
+            flip_start_side=False, indoor=False, tolerance=0.01, base_name=None):
         """Add a series of louvered Shade objects covering this Aperture.
 
         Args:
@@ -362,20 +454,27 @@ class Aperture(_BaseWithShade):
             offset: A number for the distance to louvers from this aperture.
                 Default is 0 for no offset.
             angle: A number for the for an angle to rotate the louvers in degrees.
-                Default is 0 for no rotation.
-            contour_vector: A Vector3D for the direction along which contours
-                are generated. Default is Z-Axis, which generates horizontal louvers.
+                Positive numbers indicate a downward rotation while negative numbers
+                indicate an upward rotation. Default is 0 for no rotation.
+            contour_vector: A Vector2D for the direction along which contours
+                are generated. This 2D vector will be interpreted into a 3D vector
+                within the plane of this Aperture. (0, 1) will usually generate
+                horizontal contours in 3D space, (1, 0) will generate vertical
+                contours, and (1, 1) will generate diagonal contours. Default: (0, 1).
             flip_start_side: Boolean to note whether the side the louvers start from
                 should be flipped. Default is False to have contours on top or right.
                 Setting to True will start contours on the bottom or left.
             indoor: Boolean for whether louvers should be generated facing the
                 opposite direction of the aperture normal (typically meaning
-                indoor geometry). Default: False.
+                indoor geometry). Default: 0.01, suitable for objects in meters.
             tolerance: An optional value to remove any louvers with a length less
                 than the tolerance. Default is 0, which will include all louvers
                 no matter how small.
             base_name: Optional base name for the shade objects. If None, the default
                 is InShd or OutShd depending on whether indoor is True.
+
+        Returns:
+            A list of the new Shade objects that have been generated.
         """
         angle = math.radians(angle)
         louvers = []
@@ -388,11 +487,12 @@ class Aperture(_BaseWithShade):
         else:
             shd_name_base = '{}_' + str(base_name) + '{}'
         for i, shade_geo in enumerate(shade_faces):
-            louvers.append(Shade(shd_name_base.format(self.display_name, i), shade_geo))
+            louvers.append(Shade(shd_name_base.format(self.identifier, i), shade_geo))
         if indoor:
-            self._indoor_shades.extend(louvers)
+            self.add_indoor_shades(louvers)
         else:
-            self._outdoor_shades.extend(louvers)
+            self.add_outdoor_shades(louvers)
+        return louvers
 
     def move(self, moving_vec):
         """Move this Aperture along a vector.
@@ -403,6 +503,7 @@ class Aperture(_BaseWithShade):
         """
         self._geometry = self.geometry.move(moving_vec)
         self.move_shades(moving_vec)
+        self.properties.move(moving_vec)
 
     def rotate(self, axis, angle, origin):
         """Rotate this Aperture by a certain angle around an axis and origin.
@@ -415,6 +516,7 @@ class Aperture(_BaseWithShade):
         """
         self._geometry = self.geometry.rotate(axis, math.radians(angle), origin)
         self.rotate_shades(axis, angle, origin)
+        self.properties.rotate(axis, angle, origin)
 
     def rotate_xy(self, angle, origin):
         """Rotate this Aperture counterclockwise in the world XY plane by a certain angle.
@@ -426,6 +528,7 @@ class Aperture(_BaseWithShade):
         """
         self._geometry = self.geometry.rotate_xy(math.radians(angle), origin)
         self.rotate_xy_shades(angle, origin)
+        self.properties.rotate_xy(angle, origin)
 
     def reflect(self, plane):
         """Reflect this Aperture across a plane.
@@ -436,6 +539,7 @@ class Aperture(_BaseWithShade):
         """
         self._geometry = self.geometry.reflect(plane.n, plane.o)
         self.reflect_shades(plane)
+        self.properties.reflect(plane)
 
     def scale(self, factor, origin=None):
         """Scale this Aperture by a factor from an origin point.
@@ -447,24 +551,26 @@ class Aperture(_BaseWithShade):
         """
         self._geometry = self.geometry.scale(factor, origin)
         self.scale_shades(factor, origin)
+        self.properties.scale(factor, origin)
 
-    def check_planar(self, tolerance, raise_exception=True):
+    def check_planar(self, tolerance=0.01, raise_exception=True):
         """Check whether all of the Aperture's vertices lie within the same plane.
 
         Args:
             tolerance: The minimum distance between a given vertex and a the
-                object's's plane at which the vertex is said to lie in the plane.
+                object's plane at which the vertex is said to lie in the plane.
+                Default: 0.01, suitable for objects in meters.
             raise_exception: Boolean to note whether an ValueError should be
                 raised if a vertex does not lie within the object's plane.
         """
         try:
-            return self.geometry.validate_planarity(tolerance, raise_exception)
+            return self.geometry.check_planar(tolerance, raise_exception)
         except ValueError as e:
             raise ValueError('Aperture "{}" is not planar.\n{}'.format(
                 self.display_name, e))
 
     def check_self_intersecting(self, raise_exception=True):
-        """Check whether the edges of the Aperture intersect one another (like a bowtwie)
+        """Check whether the edges of the Aperture intersect one another (like a bowtie)
 
         Args:
             raise_exception: If True, a ValueError will be raised if the object
@@ -483,7 +589,7 @@ class Aperture(_BaseWithShade):
         Args:
             tolerance: The minimum acceptable area of the object. Default is 0.0001,
                 which is equal to 1 cm2 when model units are meters. This is just
-                above the smalest size that OpenStudio will accept.
+                above the smallest size that OpenStudio will accept.
             raise_exception: If True, a ValueError will be raised if the object
                 area is below the tolerance. Default: True.
         """
@@ -502,10 +608,12 @@ class Aperture(_BaseWithShade):
         Use this method to access Writer class to write the aperture in other formats.
 
         Usage:
+
+        .. code-block:: python
+
             aperture.to.idf(aperture) -> idf string.
             aperture.to.radiance(aperture) -> Radiance string.
         """
-        raise NotImplementedError('Aperture does not yet support writing to files.')
         return writer
 
     def to_dict(self, abridged=False, included_prop=None):
@@ -513,36 +621,36 @@ class Aperture(_BaseWithShade):
 
         Args:
             abridged: Boolean to note whether the extension properties of the
-                object (ie. materials, construcitons) should be included in detail
-                (False) or just referenced by name (True). Default: False.
+                object (ie. materials, constructions) should be included in detail
+                (False) or just referenced by identifier (True). Default: False.
             included_prop: List of properties to filter keys that must be included in
                 output dictionary. For example ['energy'] will include 'energy' key if
                 available in properties to_dict. By default all the keys will be
                 included. To exclude all the keys from extensions use an empty list.
         """
         base = {'type': 'Aperture'}
-        base['name'] = self.name
+        base['identifier'] = self.identifier
         base['display_name'] = self.display_name
         base['properties'] = self.properties.to_dict(abridged, included_prop)
-
-        if 'energy' in base['properties']:
-            base['geometry'] = self._geometry.to_dict(False, True)  # enforce upper-left
-        else:
-            base['geometry'] = self._geometry.to_dict(False)
-
-        base['aperture_type'] = self.type.name
+        enforce_upper_left = True if 'energy' in base['properties'] else False
+        base['geometry'] = self._geometry.to_dict(False, enforce_upper_left)
+        base['is_operable'] = self.is_operable
         if isinstance(self.boundary_condition, Outdoors) and 'energy' in base['properties']:
             base['boundary_condition'] = self.boundary_condition.to_dict(full=True)
         else:
             base['boundary_condition'] = self.boundary_condition.to_dict()
         self._add_shades_to_dict(base, abridged, included_prop)
+        if self.user_data is not None:
+            base['user_data'] = self.user_data
         return base
 
     def __copy__(self):
-        new_ap = Aperture(self.name, self.geometry, self.type, self.boundary_condition)
+        new_ap = Aperture(self.identifier, self.geometry, self.boundary_condition,
+                          self.is_operable)
         new_ap._display_name = self.display_name
+        new_ap._user_data = None if self.user_data is None else self.user_data.copy()
         self._duplicate_child_shades(new_ap)
-        new_ap._properties.duplicate_extension_attr(self._properties)
+        new_ap._properties._duplicate_extension_attr(self._properties)
         return new_ap
 
     def __repr__(self):
